#[derive(Debug)]
pub enum Error{
	Parse(url::ParseError),
	Reqwest(reqwest::Error),
	Response(ResponseError),
	JSON(serde_json::Error),
}
impl std::fmt::Display for Error{
	fn fmt(&self,f:&mut std::fmt::Formatter<'_>)->std::fmt::Result{
		write!(f,"{self:?}")
	}
}
impl std::error::Error for Error{}

#[derive(Debug)]
pub enum SingleItemError{
	DuplicateItems,
	Other(Error),
}
impl std::fmt::Display for SingleItemError{
	fn fmt(&self,f:&mut std::fmt::Formatter<'_>)->std::fmt::Result{
		write!(f,"{self:?}")
	}
}
impl std::error::Error for SingleItemError{}

#[allow(dead_code)]
#[derive(Debug)]
pub struct StatusCodeWithUrlAndBody{
	pub status_code:reqwest::StatusCode,
	pub url:url::Url,
	pub body:String,
}

#[derive(Debug)]
pub enum ResponseError{
	Reqwest(reqwest::Error),
	StatusCodeWithUrlAndBody(StatusCodeWithUrlAndBody),
}
impl std::fmt::Display for ResponseError{
	fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
		write!(f,"{self:?}")
	}
}
impl std::error::Error for ResponseError{}
// lazy function to draw out meaningful info from http response on failure
pub async fn response_ok(response:reqwest::Response)->Result<reqwest::Response,ResponseError>{
	let status_code=response.status();
	if status_code.is_success(){
		Ok(response)
	}else{
		let url=response.url().to_owned();
		let bytes=response.bytes().await.map_err(ResponseError::Reqwest)?;
		let body=String::from_utf8_lossy(&bytes).to_string();
		Err(ResponseError::StatusCodeWithUrlAndBody(StatusCodeWithUrlAndBody{
			status_code,
			url,
			body,
		}))
	}
}

#[derive(Clone,Copy,PartialEq,Eq,serde::Serialize,serde::Deserialize)]
pub struct ScriptID(pub(crate)i64);
#[derive(Clone,Copy,serde::Serialize,serde::Deserialize)]
pub struct ScriptPolicyID(pub(crate)i64);

#[allow(nonstandard_style)]
pub struct GetScriptRequest{
	pub ScriptID:ScriptID,
}
#[allow(nonstandard_style)]
#[derive(serde::Serialize)]
pub struct GetScriptsRequest<'a>{
	pub Page:u32,
	pub Limit:u32,
	#[serde(skip_serializing_if="Option::is_none")]
	pub Name:Option<&'a str>,
	#[serde(skip_serializing_if="Option::is_none")]
	pub Hash:Option<&'a str>,
	#[serde(skip_serializing_if="Option::is_none")]
	pub Source:Option<&'a str>,
	#[serde(skip_serializing_if="Option::is_none")]
	pub SubmissionID:Option<i64>,
}
pub struct HashRequest<'a>{
	pub hash:&'a str,
}
#[allow(nonstandard_style)]
#[derive(serde::Deserialize)]
pub struct ScriptResponse{
	pub ID:ScriptID,
	pub Name:String,
	pub Hash:String,
	pub Source:String,
	pub SubmissionID:i64,
}
#[allow(nonstandard_style)]
#[derive(serde::Serialize)]
pub struct CreateScriptRequest<'a>{
	pub Name:&'a str,
	pub Source:&'a str,
	#[serde(skip_serializing_if="Option::is_none")]
	pub SubmissionID:Option<i64>,
}
#[allow(nonstandard_style)]
#[derive(serde::Deserialize)]
pub struct ScriptIDResponse{
	pub ID:ScriptID,
}

#[derive(PartialEq,Eq,serde_repr::Serialize_repr,serde_repr::Deserialize_repr)]
#[repr(i32)]
pub enum Policy{
	None=0, // not yet reviewed
	Allowed=1,
	Blocked=2,
	Delete=3,
	Replace=4,
}

#[allow(nonstandard_style)]
#[derive(serde::Serialize)]
pub struct GetScriptPoliciesRequest<'a>{
	pub Page:u32,
	pub Limit:u32,
	#[serde(skip_serializing_if="Option::is_none")]
	pub FromScriptHash:Option<&'a str>,
	#[serde(skip_serializing_if="Option::is_none")]
	pub ToScriptID:Option<ScriptID>,
	#[serde(skip_serializing_if="Option::is_none")]
	pub Policy:Option<Policy>,
}
#[allow(nonstandard_style)]
#[derive(serde::Deserialize)]
pub struct ScriptPolicyResponse{
	pub ID:ScriptPolicyID,
	pub FromScriptHash:String,
	pub ToScriptID:ScriptID,
	pub Policy:Policy
}
#[allow(nonstandard_style)]
#[derive(serde::Serialize)]
pub struct CreateScriptPolicyRequest{
	pub FromScriptID:ScriptID,
	pub ToScriptID:ScriptID,
	pub Policy:Policy,
}
#[allow(nonstandard_style)]
#[derive(serde::Deserialize)]
pub struct ScriptPolicyIDResponse{
	pub ID:ScriptPolicyID,
}

#[allow(nonstandard_style)]
#[derive(serde::Serialize)]
pub struct UpdateScriptPolicyRequest{
	pub ID:ScriptPolicyID,
	#[serde(skip_serializing_if="Option::is_none")]
	pub FromScriptID:Option<ScriptID>,
	#[serde(skip_serializing_if="Option::is_none")]
	pub ToScriptID:Option<ScriptID>,
	#[serde(skip_serializing_if="Option::is_none")]
	pub Policy:Option<Policy>,
}

#[allow(nonstandard_style)]
pub struct UpdateSubmissionModelRequest{
	pub SubmissionID:i64,
	pub ModelID:u64,
	pub ModelVersion:u64,
}

#[allow(nonstandard_style)]
pub struct ActionSubmissionUploadedRequest{
	pub SubmissionID:i64,
	pub TargetAssetID:Option<u64>,
}

pub struct SubmissionID(pub i64);
